@@ -13,7 +13,7 @@ use {
     crate::{
         cluster_info::{Ping, CRDS_UNIQUE_PUBKEY_CAPACITY},
         contact_info::ContactInfo,
-        crds::{Crds, VersionedCrdsValue},
+        crds::{Crds, GossipRoute, VersionedCrdsValue},
         crds_gossip::{get_stake, get_weight},
         crds_gossip_error::CrdsGossipError,
         crds_value::CrdsValue,
@@ -333,7 +333,7 @@ impl CrdsGossipPull {
     {
         for caller in callers {
             let key = caller.pubkey();
-            let _ = crds.insert(caller, now);
+            let _ = crds.insert(caller, now, GossipRoute::PullRequest);
             crds.update_record_timestamp(&key, now);
         }
     }
@@ -413,11 +413,11 @@ impl CrdsGossipPull {
     ) {
         let mut owners = HashSet::new();
         for response in responses_expired_timeout {
-            let _ = crds.insert(response, now);
+            let _ = crds.insert(response, now, GossipRoute::PullResponse);
         }
         for response in responses {
             let owner = response.pubkey();
-            if let Ok(()) = crds.insert(response, now) {
+            if let Ok(()) = crds.insert(response, now, GossipRoute::PullResponse) {
                 stats.success += 1;
                 self.num_pulls += 1;
                 owners.insert(owner);
@@ -688,14 +688,16 @@ pub(crate) mod tests {
             &solana_sdk::pubkey::new_rand(),
             0,
         )));
-        crds.insert(me.clone(), 0).unwrap();
+        crds.insert(me.clone(), 0, GossipRoute::LocalMessage)
+            .unwrap();
         for i in 1..=30 {
             let entry = CrdsValue::new_unsigned(CrdsData::ContactInfo(ContactInfo::new_localhost(
                 &solana_sdk::pubkey::new_rand(),
                 0,
             )));
             let id = entry.label().pubkey();
-            crds.insert(entry.clone(), 0).unwrap();
+            crds.insert(entry.clone(), 0, GossipRoute::LocalMessage)
+                .unwrap();
             stakes.insert(id, i * 100);
         }
         let now = 1024;
@@ -750,10 +752,14 @@ pub(crate) mod tests {
             ..ContactInfo::default()
         }));
 
-        crds.insert(me.clone(), 0).unwrap();
-        crds.insert(spy.clone(), 0).unwrap();
-        crds.insert(node_123.clone(), 0).unwrap();
-        crds.insert(node_456.clone(), 0).unwrap();
+        crds.insert(me.clone(), 0, GossipRoute::LocalMessage)
+            .unwrap();
+        crds.insert(spy.clone(), 0, GossipRoute::LocalMessage)
+            .unwrap();
+        crds.insert(node_123.clone(), 0, GossipRoute::LocalMessage)
+            .unwrap();
+        crds.insert(node_456.clone(), 0, GossipRoute::LocalMessage)
+            .unwrap();
 
         // shred version 123 should ignore nodes with versions 0 and 456
         let options = node
@@ -811,8 +817,10 @@ pub(crate) mod tests {
             ..ContactInfo::default()
         }));
 
-        crds.insert(me.clone(), 0).unwrap();
-        crds.insert(node_123.clone(), 0).unwrap();
+        crds.insert(me.clone(), 0, GossipRoute::LocalMessage)
+            .unwrap();
+        crds.insert(node_123.clone(), 0, GossipRoute::LocalMessage)
+            .unwrap();
 
         // Empty gossip_validators -- will pull from nobody
         let mut gossip_validators = HashSet::new();
@@ -914,7 +922,10 @@ pub(crate) mod tests {
         for _ in 0..40_000 {
             let keypair = keypairs.choose(&mut rng).unwrap();
             let value = CrdsValue::new_rand(&mut rng, Some(keypair));
-            if crds.insert(value, rng.gen()).is_ok() {
+            if crds
+                .insert(value, rng.gen(), GossipRoute::LocalMessage)
+                .is_ok()
+            {
                 num_inserts += 1;
             }
         }
@@ -973,7 +984,7 @@ pub(crate) mod tests {
             Err(CrdsGossipError::NoPeers)
         );
 
-        crds.insert(entry, 0).unwrap();
+        crds.insert(entry, 0, GossipRoute::LocalMessage).unwrap();
         assert_eq!(
             node.new_pull_request(
                 &thread_pool,
@@ -997,7 +1008,8 @@ pub(crate) mod tests {
             .unwrap()
             .mock_pong(new.id, new.gossip, Instant::now());
         let new = CrdsValue::new_unsigned(CrdsData::ContactInfo(new));
-        crds.insert(new.clone(), now).unwrap();
+        crds.insert(new.clone(), now, GossipRoute::LocalMessage)
+            .unwrap();
         let req = node.new_pull_request(
             &thread_pool,
             &crds,
@@ -1017,7 +1029,8 @@ pub(crate) mod tests {
         node.mark_pull_request_creation_time(new.contact_info().unwrap().id, now);
         let offline = ContactInfo::new_localhost(&solana_sdk::pubkey::new_rand(), now);
         let offline = CrdsValue::new_unsigned(CrdsData::ContactInfo(offline));
-        crds.insert(offline, now).unwrap();
+        crds.insert(offline, now, GossipRoute::LocalMessage)
+            .unwrap();
         let req = node.new_pull_request(
             &thread_pool,
             &crds,
@@ -1052,15 +1065,17 @@ pub(crate) mod tests {
             0,
         )));
         let mut node = CrdsGossipPull::default();
-        crds.insert(entry, now).unwrap();
+        crds.insert(entry, now, GossipRoute::LocalMessage).unwrap();
         let old = ContactInfo::new_localhost(&solana_sdk::pubkey::new_rand(), 0);
         ping_cache.mock_pong(old.id, old.gossip, Instant::now());
         let old = CrdsValue::new_unsigned(CrdsData::ContactInfo(old));
-        crds.insert(old.clone(), now).unwrap();
+        crds.insert(old.clone(), now, GossipRoute::LocalMessage)
+            .unwrap();
         let new = ContactInfo::new_localhost(&solana_sdk::pubkey::new_rand(), 0);
         ping_cache.mock_pong(new.id, new.gossip, Instant::now());
         let new = CrdsValue::new_unsigned(CrdsData::ContactInfo(new));
-        crds.insert(new.clone(), now).unwrap();
+        crds.insert(new.clone(), now, GossipRoute::LocalMessage)
+            .unwrap();
 
         // set request creation time to now.
         let now = now + 50_000;
@@ -1145,11 +1160,13 @@ pub(crate) mod tests {
         )));
         let caller = entry.clone();
         let node = CrdsGossipPull::default();
-        node_crds.insert(entry, 0).unwrap();
+        node_crds
+            .insert(entry, 0, GossipRoute::LocalMessage)
+            .unwrap();
         let new = ContactInfo::new_localhost(&solana_sdk::pubkey::new_rand(), 0);
         ping_cache.mock_pong(new.id, new.gossip, Instant::now());
         let new = CrdsValue::new_unsigned(CrdsData::ContactInfo(new));
-        node_crds.insert(new, 0).unwrap();
+        node_crds.insert(new, 0, GossipRoute::LocalMessage).unwrap();
         let mut pings = Vec::new();
         let req = node.new_pull_request(
             &thread_pool,
@@ -1183,7 +1200,11 @@ pub(crate) mod tests {
             CRDS_GOSSIP_PULL_MSG_TIMEOUT_MS,
         )));
         dest_crds
-            .insert(new, CRDS_GOSSIP_PULL_MSG_TIMEOUT_MS)
+            .insert(
+                new,
+                CRDS_GOSSIP_PULL_MSG_TIMEOUT_MS,
+                GossipRoute::LocalMessage,
+            )
             .unwrap();
 
         //should skip new value since caller is to old
@@ -1232,7 +1253,9 @@ pub(crate) mod tests {
         )));
         let caller = entry.clone();
         let node = CrdsGossipPull::default();
-        node_crds.insert(entry, 0).unwrap();
+        node_crds
+            .insert(entry, 0, GossipRoute::LocalMessage)
+            .unwrap();
         let mut ping_cache = PingCache::new(
             Duration::from_secs(20 * 60), // ttl
             128,                          // capacity
@@ -1240,7 +1263,7 @@ pub(crate) mod tests {
         let new = ContactInfo::new_localhost(&solana_sdk::pubkey::new_rand(), 0);
         ping_cache.mock_pong(new.id, new.gossip, Instant::now());
         let new = CrdsValue::new_unsigned(CrdsData::ContactInfo(new));
-        node_crds.insert(new, 0).unwrap();
+        node_crds.insert(new, 0, GossipRoute::LocalMessage).unwrap();
         let mut pings = Vec::new();
         let req = node.new_pull_request(
             &thread_pool,
@@ -1287,7 +1310,9 @@ pub(crate) mod tests {
         let caller = entry.clone();
         let node_pubkey = entry.label().pubkey();
         let mut node = CrdsGossipPull::default();
-        node_crds.insert(entry, 0).unwrap();
+        node_crds
+            .insert(entry, 0, GossipRoute::LocalMessage)
+            .unwrap();
         let mut ping_cache = PingCache::new(
             Duration::from_secs(20 * 60), // ttl
             128,                          // capacity
@@ -1295,14 +1320,16 @@ pub(crate) mod tests {
         let new = ContactInfo::new_localhost(&solana_sdk::pubkey::new_rand(), 1);
         ping_cache.mock_pong(new.id, new.gossip, Instant::now());
         let new = CrdsValue::new_unsigned(CrdsData::ContactInfo(new));
-        node_crds.insert(new, 0).unwrap();
+        node_crds.insert(new, 0, GossipRoute::LocalMessage).unwrap();
 
         let mut dest_crds = Crds::default();
         let new_id = solana_sdk::pubkey::new_rand();
         let new = ContactInfo::new_localhost(&new_id, 1);
         ping_cache.mock_pong(new.id, new.gossip, Instant::now());
         let new = CrdsValue::new_unsigned(CrdsData::ContactInfo(new));
-        dest_crds.insert(new.clone(), 0).unwrap();
+        dest_crds
+            .insert(new.clone(), 0, GossipRoute::LocalMessage)
+            .unwrap();
 
         // node contains a key from the dest node, but at an older local timestamp
         let same_key = ContactInfo::new_localhost(&new_id, 0);
@@ -1310,7 +1337,9 @@ pub(crate) mod tests {
         let same_key = CrdsValue::new_unsigned(CrdsData::ContactInfo(same_key));
         assert_eq!(same_key.label(), new.label());
         assert!(same_key.wallclock() < new.wallclock());
-        node_crds.insert(same_key.clone(), 0).unwrap();
+        node_crds
+            .insert(same_key.clone(), 0, GossipRoute::LocalMessage)
+            .unwrap();
         assert_eq!(node_crds.get(&same_key.label()).unwrap().local_timestamp, 0);
         let mut done = false;
         let mut pings = Vec::new();
@@ -1383,12 +1412,16 @@ pub(crate) mod tests {
         let node_label = entry.label();
         let node_pubkey = node_label.pubkey();
         let node = CrdsGossipPull::default();
-        node_crds.insert(entry, 0).unwrap();
+        node_crds
+            .insert(entry, 0, GossipRoute::LocalMessage)
+            .unwrap();
         let old = CrdsValue::new_unsigned(CrdsData::ContactInfo(ContactInfo::new_localhost(
             &solana_sdk::pubkey::new_rand(),
             0,
         )));
-        node_crds.insert(old.clone(), 0).unwrap();
+        node_crds
+            .insert(old.clone(), 0, GossipRoute::LocalMessage)
+            .unwrap();
         let value_hash = node_crds.get(&old.label()).unwrap().value_hash;
 
         //verify self is valid
